@@ -16,13 +16,18 @@
 These tests are to make sure we do something sane in the presence of
 classic ExtensionClass classes and instances.
 """
-import doctest
 import unittest
 
 from zope.interface.tests import odd
-from zope.interface import Interface, implements, classProvides
-from zope.interface import directlyProvides, providedBy, directlyProvidedBy
-from zope.interface import classImplements, classImplementsOnly, implementedBy
+from zope.interface import Interface
+from zope.interface import implementer
+from zope.interface import directlyProvides
+from zope.interface import providedBy
+from zope.interface import directlyProvidedBy
+from zope.interface import classImplements
+from zope.interface import classImplementsOnly
+from zope.interface import implementedBy
+from zope.interface._compat import _skip_under_py3k
 
 class I1(Interface): pass
 class I2(Interface): pass
@@ -117,11 +122,14 @@ class Test(unittest.TestCase):
         self.failUnless(providedBy(c).extends(I5))
 
     def test_classImplements(self):
-        class A(Odd):
-            implements(I3)
 
+        @implementer(I3)
+        class A(Odd):
+            pass
+
+        @implementer(I4)
         class B(Odd):
-            implements(I4)
+            pass
 
         class C(A, B):
             pass
@@ -133,11 +141,13 @@ class Test(unittest.TestCase):
                          ['I1', 'I2', 'I5', 'I3', 'I4'])
 
     def test_classImplementsOnly(self):
+        @implementer(I3)
         class A(Odd):
-            implements(I3)
+            pass
 
+        @implementer(I4)
         class B(Odd):
-            implements(I4)
+            pass
 
         class C(A, B):
             pass
@@ -180,19 +190,20 @@ class Test(unittest.TestCase):
         directlyProvides(ob, directlyProvidedBy(ob), I2)
         self.failUnless(I2 in providedBy(ob))
 
+    @_skip_under_py3k
     def test_directlyProvides_fails_for_odd_class(self):
         self.assertRaises(TypeError, directlyProvides, C, I5)
 
     # see above
-    def TODO_test_classProvides_fails_for_odd_class(self):
-        try:
-            class A(Odd):
-                classProvides(I1)
-        except TypeError:
-            pass # Sucess
-        self.failUnless(False,
-                     "Shouldn't be able to use directlyProvides on odd class."
-                     )
+    #def TODO_test_classProvides_fails_for_odd_class(self):
+    #    try:
+    #        class A(Odd):
+    #            classProvides(I1)
+    #    except TypeError:
+    #        pass # Sucess
+    #    self.assert_(False,
+    #                 "Shouldn't be able to use directlyProvides on odd class."
+    #                 )
 
     def test_implementedBy(self):
         class I2(I1): pass
@@ -208,15 +219,9 @@ class Test(unittest.TestCase):
         self.assertEqual([i.getName() for i in implementedBy(C2)],
                          ['I3', 'I2'])
 
-
-
-
 def test_suite():
+    import doctest
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(Test))
     suite.addTest(doctest.DocTestSuite(odd))
     return suite
-
-
-if __name__ == '__main__':
-    unittest.main()
